/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.guete.GWert;
import de.bsvrz.dua.guete.GueteException;
import de.bsvrz.dua.guete.GueteVerfahren;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Ausrechnet Die Summe aller Fahrzeugen.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class AggregatorFahrzeugAnzahl extends AbstraktAggregator {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * ATG String Konstante.
	 */
	private static final String ATG_LZD_FZ_ANZAHL = "atg.langZeitDatenFahrzeugAnzahl";

	/**
	 * Erweitert die Klasse LZZDAten um die Summe der Anzahl aller FZ.
	 *
	 */
	private class LZZfzAnzahlDaten extends LZZDaten {
		/**
		 * Die Summe des Anzahls der FZ.
		 */
		private final long[] summeAnzahl = new long[AbstraktAggregator.FZ_ATTRIB.length];

		/**
		 * Standarkonsturktor.
		 */
		LZZfzAnzahlDaten() {
			resetDaten();
		}

		/**
		 * Initialisiert die Daten.
		 */
		@Override
		public void resetDaten() {
			super.resetDaten();
			for (int i = 0; i < summeAnzahl.length; i++) {
				summeAnzahl[i] = 0;

			}
		}

		/**
		 * Ergibt die Summe der FZ.
		 *
		 * @return die Summe
		 */
		public long[] getSummeAnzahl() {
			return summeAnzahl;
		}
	};

	@Override
	public void bearbeiteDatenSatz(final SystemObject objekt,
			final Data datensatz, final long zeitStempel) {

		if (datensatz == null) {
			return;
		}
		final LZZfzAnzahlDaten lzz = (LZZfzAnzahlDaten) getLzzDatenForSystemObject(objekt);
		final Data summeDatensatz = lzz.getErgebnissDatenSatz();
		if (summeDatensatz == null) {
			return;
		}

		lzz.incrementAnzahlDatenSaetze();

		for (int i = 0; i < AbstraktAggregator.FZ_ATTRIB.length; i++) {
			final Data subData = datensatz.getItem("q"
					+ AbstraktAggregator.FZ_ATTRIB[i]);
			final Data subSummeData = summeDatensatz
					.getItem(AbstraktAggregator.FZ_ATTRIB[i]);
			final long[] summeAnzahl = lzz.getSummeAnzahl();
			long wert = subData.getUnscaledValue(AbstraktAggregator.STR_WERT)
					.longValue();
			final boolean[] ermittelbarWertEingekommen = lzz
					.getErmittelbarWert();

			if (wert < 0) {
				wert = 0;
			} else {
				ermittelbarWertEingekommen[i] = true;
			}

			summeAnzahl[i] += wert;

			if (lzz.getAnzahlDatenSaetze() == 1) {
				copyAttribute(subData, subSummeData);
			} else {
				try {
					final GWert summeGuete;
					summeGuete = GueteVerfahren.summe(GueteVerfahren.gewichte(
							new GWert(datensatz, "q"
									+ AbstraktAggregator.FZ_ATTRIB[i]),
							lzz.getAnzahlDatenSaetze() - 1), GueteVerfahren
							.gewichte(new GWert(summeDatensatz,
									AbstraktAggregator.FZ_ATTRIB[i]), 1));
					summeGuete.exportiere(subSummeData
							.getItem(AbstraktAggregator.STR_GUETE));
				} catch (final GueteException e) {
					AggregatorFahrzeugAnzahl.LOGGER
					.error("Fehler bei Ausrechnen der Guete fuer aggregierte Daten "
							+ e.getMessage());
					e.printStackTrace();
				}
			}

		}
	}

	@Override
	public void erzeugeErgebniss(final SystemObject objekt,
			final long zeitStempelVon, final long zeitStempelBis,
			final boolean ermittelbar) {

		final LZZfzAnzahlDaten lzz = (LZZfzAnzahlDaten) getLzzDatenForSystemObject(objekt);
		final Data ergebnissDatenSatz = lzz.getErgebnissDatenSatz();

		if (lzz.getZeitStempelLetztesDS() >= zeitStempelVon) {
			setLeer(ergebnissDatenSatz);
			lzz.setAnzahlDatenSaetze(0);
			lzz.resetDaten();
			return;
		}
		final long[] summe = lzz.getSummeAnzahl();
		final boolean[] ermittelbarWertEingekommen = lzz.getErmittelbarWert();
		for (int i = 0; i < AbstraktAggregator.FZ_ATTRIB.length; i++) {
			if (ermittelbarWertEingekommen[i]) {
				ergebnissDatenSatz.getItem(AbstraktAggregator.FZ_ATTRIB[i])
				.getUnscaledValue(AbstraktAggregator.STR_WERT)
				.set(summe[i]);
			} else {
				setAttributNichtErmittelbar(ergebnissDatenSatz
						.getItem(AbstraktAggregator.FZ_ATTRIB[i]));
			}
		}

		if (!ermittelbar) {
			setNichtErmittelbar(ergebnissDatenSatz);
		}

		final ResultData result = new ResultData(objekt, getDdAggregierteLzd(),
				zeitStempelVon, ergebnissDatenSatz, true);

		sendeDatenSatz(objekt, result);

		setLeer(ergebnissDatenSatz);
		lzz.setAnzahlDatenSaetze(0);
		lzz.resetDaten();
		lzz.setZeitStempelLetztesDS(zeitStempelVon);
	}

	@Override
	public DataDescription init(final Aspect asp, final ClientDavInterface dav,
			final Collection<SystemObject> objekteLZZ) {

		resetLzzDaten();
		final DataDescription dd = new DataDescription(dav.getDataModel()
				.getAttributeGroup(AggregatorFahrzeugAnzahl.ATG_LZD_FZ_ANZAHL),
				asp);

		for (final SystemObject lzz : objekteLZZ) {
			final LZZDaten lzzStruct = new LZZfzAnzahlDaten();
			final Data data = dav.createData(dav.getDataModel()
					.getAttributeGroup(
							AggregatorFahrzeugAnzahl.ATG_LZD_FZ_ANZAHL));
			lzzStruct.setErgebnissDatenSatz(data);
			setLeer(data);
			addLzzDaten(lzz, lzzStruct);
		}
		return dd;
	}
}
