/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.LinkedList;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import com.bitctrl.Constants;

import de.bsvrz.dua.abfrpuffer.Ereignis.EreignissTyp;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;

/**
 * Testet die SWE analog PruefSpez.
 *
 * @author BitCtrl Systems GmbH, Thierfelder
 */
@Ignore("Einbidung des Testsytems prüfen")
public class TestNachPruefSpez extends AbfragePufferDatenTester {

	/**
	 * Vorbereitung der Zeit- und SM-Simulation.
	 *
	 * @throws Exception
	 *             wird weitergereicht.
	 */
	@Override
	@Before
	public void prepare() throws Exception {

		System.out
				.println("*** Vorbereitung der Zeit- und SM-Simulation ***\n");
		System.out
				.println("Steuermodulsimulator fuer Steuermodul \"sm\" wird initialisiert:\n"
						+ "DE-LVE: de.lve.1, de.lve.2, de.lve.3\n"
						+ "DE-AXL: de.axl.1, de.axl.2, de.axl.3\n");

		final int lveIntervall = 1;
		final int axlIntervall = 1;

		final LinkedList<DePid> listLve = new LinkedList<>();
		final LinkedList<DePid> listAxl = new LinkedList<>();

		listLve.add(new DePid("de.lve.1", "fs.1", lveIntervall));
		listLve.add(new DePid("de.lve.2", "fs.2", lveIntervall));
		listLve.add(new DePid("de.lve.3", "fs.3", lveIntervall));
		listAxl.add(new DePid("de.axl.1", "axlms.1", axlIntervall));
		listAxl.add(new DePid("de.axl.2", "axlms.2", axlIntervall));
		listAxl.add(new DePid("de.axl.3", "axlms.3", axlIntervall));
		getSteuerModuleSim().add(
				new SteuerModulSim(listAxl, "cluster.axl", listLve,
						"cluster.lve"));

		// Die default Werte im DAV, sonst wird es dem Test nicht bestehen
		// Abfrage Modul muss so parametriert werden !
		getAxlOffset()[getParamLen() - 1] = 1;
		getAxlZyklus()[getParamLen() - 1] = 1;

		getLveOffset()[getParamLen() - 1] = 1;
		getLveZyklus()[getParamLen() - 1] = 1;

		try {
			Thread.sleep(100L);
		} catch (final Exception e) {
			// TODO handle exception
		}
	}

	/**
	 * Testfall analog PruefSpez.
	 */
	@Test
	public synchronized void testNachPruefSpez() {
		try {
			final String zeitText = "01.01.2006 00:00:00.000";

			final DateFormat datumFormat = new SimpleDateFormat(
					DUAKonstanten.ZEIT_FORMAT_GENAU_STR);

			final long zeit = datumFormat.parse(zeitText).getTime();

			System.out
					.println("Setze letzen empfangenen Zeitstempel fuer Steuermodul: 01.01.2006 00:00:00.000");

			SteuerModulSim.setAeltesterLveZeitStempellVorhanden(
					getSteuerModuleSim(), zeit);
			SteuerModulSim.setAeltesterAxlZeitStempellVorhanden(
					getSteuerModuleSim(), zeit);

			ZeitSimulator.start(zeitText, 10000, 8000);
			StandardApplicationRunner.run(this, Verbindung.CON_DATA.clone());

			// ZeitSimulator.schedule(new Ereignis(SteuerModulSim
			// .getDe("de.lve.1"),
			// EreignissTyp.EREIGNIS_SM_OFFLINE),
			// "01.01.2006 4:30:00.000");
			//
			// ZeitSimulator.schedule(new Ereignis(SteuerModulSim
			// .getDe("de.lve.1"),
			// EreignissTyp.EREIGNIS_SM_GEHT_ONLINE),
			// "01.01.2006 6:30:00.000");
			//
			// ZeitSimulator.schedule(new Ereignis(SteuerModulSim
			// .getDe("de.axl.1"),
			// EreignissTyp.EREIGNIS_SM_OFFLINE),
			// "01.01.2006 4:30:00.000");
			//
			// ZeitSimulator.schedule(new Ereignis(SteuerModulSim
			// .getDe("de.axl.1"),
			// EreignissTyp.EREIGNIS_SM_GEHT_ONLINE),
			// "01.01.2006 6:30:00.000");

			final ArrayList<DeSim> list = new ArrayList<>();
			list.add(SteuerModulSim.getDe("de.lve.1"));
			list.add(SteuerModulSim.getDe("de.axl.1"));

			ZeitSimulator
					.schedule(new Ereignis(list,
							EreignissTyp.EREIGNIS_SM_OFFLINE),
							"01.01.2006 4:30:00.000");

			ZeitSimulator.schedule(new Ereignis(list,
					EreignissTyp.EREIGNIS_SM_GEHT_ONLINE),
					"01.01.2006 6:30:00.000");

			ZeitSimulator.schedule(
					new Ereignis(SteuerModulSim.getDe("de.lve.1"),
							EreignissTyp.EREIGNIS_DE_WIRD_NICHT_ARCHIVIERT),
					"01.01.2006 10:30:00.000");

			ZeitSimulator.schedule(
					new Ereignis(SteuerModulSim.getDe("de.lve.1"),
							EreignissTyp.EREIGNIS_DE_WIRD_ARCHIVIERT),
					"01.01.2006 13:30:00.000");

			ZeitSimulator.schedule(
					new Ereignis(SteuerModulSim.getDe("de.axl.1"),
							EreignissTyp.EREIGNIS_DE_WIRD_NICHT_ARCHIVIERT),
					"01.01.2006 12:30:00.000");

			ZeitSimulator.schedule(
					new Ereignis(SteuerModulSim.getDe("de.axl.1"),
							EreignissTyp.EREIGNIS_DE_WIRD_ARCHIVIERT),
					"01.01.2006 14:30:00.000");

			this.wait();

		} catch (final Exception e) {
			getDav().disconnect(true, "Fehler bei Test: testNachPruefSpez()");
			System.out.println("\n***" + e.getMessage() + "***\n");
			e.printStackTrace();
		}

		getDav().disconnect(false, Constants.EMPTY_STRING);
	}

}
