/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.bitctrl.Constants;

import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;
import de.bsvrz.sys.funclib.bitctrl.dua.ObjektWecker;

/**
 * Ermoeglicht die Zeit schneller zu machen, damit die Testsimulation nicht in
 * reelen Zeit laufen muss
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public class ZeitSimulator {

	/**
	 * Der Beginnzeitpunkt in reelen Zeit
	 */
	private static long zeitpunktBeginnEcht;
	/**
	 * Die zum Beginn passende "virtuelle Zeit"
	 */
	private static long zeitpunktBeginnVirtuell;
	/**
	 * Aktuelle zeit (wird bei nicht fliessenden Simulation benutzt, mit
	 * Zeitsprungen)
	 */
	private static long zeitpunktJetzt;
	/**
	 * Der Wecker kann Eriegnisse um bestimmer virtuellen Zeit aufrufen
	 */
	private static ObjektWecker wecker = new ObjektWecker();
	/**
	 * Laenge des Timeoutintervalls in der Simulation
	 */
	private static long simTimeout = 8000;
	/**
	 * der Multiplikator ermoeglicht die Zeit zu verschnellen
	 *
	 */
	//
	// wird mit 1000 dividiert, so 1000 beduetet 1:1
	// 500 ist 0.5:1 (zweimal langsamer)
	// 2000 ist 2:1 (zweimal schneller) usw.
	//
	// virtuell = echt * multiplier / 1000
	//
	private static long multiplikator = 1000;

	/**
	 * Ob die Simulation fliessend lauft (ohne Zeitsprunge)
	 */
	private static boolean lauft;
	/**
	 * Wenn der Zeitsimulatior offlline sind, dann die Zeit fliesst normal
	 */
	private static boolean offline = true;

	/**
	 * Startet den Zeitsimulator
	 *
	 * @param zeitpunktBeginntEcht
	 *            echter Zeitpunkt am Beginn
	 * @param zeitpunktBeginntVirtuell
	 *            virtueller Zeitpunkt am Beginn
	 * @param multiplier
	 *            Geschwindigkeit des Zeitablaufs
	 */
	static synchronized void start(final long zeitpunktBeginntEcht, final long zeitpunktBeginntVirtuell,
			final long multiplier) {
		ZeitSimulator.zeitpunktBeginnEcht = zeitpunktBeginntEcht;
		ZeitSimulator.zeitpunktBeginnVirtuell = zeitpunktBeginntVirtuell;
		ZeitSimulator.multiplikator = multiplier;
		ZeitSimulator.lauft = true;
		ZeitSimulator.offline = false;
	}

	/**
	 * Startet den Zeitsimulator
	 *
	 * @param zeitPunktBeginntEcht
	 *            echter Zeitpunkt am Beginn
	 * @param zeitPunktVirtuell
	 *            virtueller Zeitpunkt am Beginn
	 * @param multiplier
	 *            Geschwindigkeit des Zeitablaufs
	 */
	static synchronized void start(final long zeitPunktBeginntEcht, final String zeitPunktVirtuell,
			final long multiplier) {

		ZeitSimulator.zeitpunktBeginnEcht = zeitPunktBeginntEcht;
		try {
			final DateFormat datumFormat = new SimpleDateFormat(DUAKonstanten.ZEIT_FORMAT_GENAU_STR);

			ZeitSimulator.zeitpunktBeginnVirtuell = datumFormat.parse(zeitPunktVirtuell).getTime();
		} catch (final Exception e) {
			e.printStackTrace();
		}
		ZeitSimulator.multiplikator = multiplier;
		ZeitSimulator.lauft = true;
		ZeitSimulator.offline = false;
	}

	/**
	 * Startet den Zeitsimulator, als reeler Zeitpunkt wird die aktuelle
	 * Systemzeit benutzt
	 *
	 * @param zeitPunktVirtuell
	 *            virtueller Zeitpunkt am Beginn
	 * @param virtStundeInReelMs
	 *            Wie lange dauert eine virtuelle Stunde in Realwelt
	 */
	static synchronized void start(final String zeitPunktVirtuell, final long virtStundeInReelMs, final long timeout) {
		System.out.println("Starte Zeitsimulator: " + zeitPunktVirtuell + "\n" + "  Beschl.-Faktor: "
				+ (Constants.MILLIS_PER_HOUR / virtStundeInReelMs) + "\n  " + "Dauer einer Stunde: "
				+ virtStundeInReelMs + "ms");
		ZeitSimulator.simTimeout = timeout;
		final long multiplier = (AbfrPufferKonstanten.STUNDE_IN_MS * 1000) / virtStundeInReelMs;
		ZeitSimulator.start(System.currentTimeMillis(), zeitPunktVirtuell, multiplier);
	}

	/**
	 * Stoppt die Simulation
	 */
	static synchronized void stop() {
		ZeitSimulator.lauft = false;
	}

	/**
	 * Stellt die Zeit auf einen bestimmten Wert fest darf nur dann gemacht
	 * werden, wenn die Simulation nicht Lauft sonst der Wecker wird in Flaschen
	 * Zeit signalisieren
	 *
	 * @param time
	 */
	static synchronized void setTime(final String time) {
		try {
			final DateFormat datumFormat = new SimpleDateFormat(DUAKonstanten.ZEIT_FORMAT_GENAU_STR);

			ZeitSimulator.zeitpunktJetzt = datumFormat.parse(time).getTime();
		} catch (final Exception e) {
			e.printStackTrace();
		}
		ZeitSimulator.offline = false;
	}

	/**
	 * Stellt die Aktuelle virtuelle Zeit fest ZeitSprunge duerfen nur dann
	 * gemacht werden, wenn die Simulation nicht Lauft sonst der Wecker wird in
	 * Flaschen Zeit signalisieren
	 *
	 * @param zeitIntervall
	 */
	static synchronized void zeitSprung(final long zeitIntervall) {
		if (!ZeitSimulator.lauft) {
			ZeitSimulator.zeitpunktBeginnVirtuell += zeitIntervall;
		}
	}

	/**
	 * Stellt die Zeit auf einen bestimmten Wert fest darf nur dann gemacht
	 * werden, wenn die Simulation nicht Lauft sonst der Wecker wird in Flaschen
	 * Zeit signalisieren
	 *
	 * @param time
	 */
	static synchronized void setTimeMillis(final long time) {
		ZeitSimulator.zeitpunktJetzt = time;
	}

	/**
	 * Erfragt die aktuelle virtuelle Zeit, wenn die Simulation nicht fliessend
	 * lauft, dann gibt die letzte eingestellte Zeit, wenn der Simulatior
	 * offline ist, dann gibt die Systemzeit
	 *
	 * @return Zeit
	 */
	static synchronized long getTimeMillis() {
		if (ZeitSimulator.lauft) {
			return (((System.currentTimeMillis() - ZeitSimulator.zeitpunktBeginnEcht) * ZeitSimulator.multiplikator)
					/ 1000) + ZeitSimulator.zeitpunktBeginnVirtuell;
		} else if (ZeitSimulator.offline) {
			return System.currentTimeMillis();
		} else {
			return ZeitSimulator.zeitpunktJetzt;
		}

	}

	/**
	 * konvertiert einen ZeitPunkt in virtueller Zeit in reelle
	 *
	 * @param zeit
	 *            Virtuelle Zeit
	 * @return reeller Zeit
	 */
	static long getVirtuelleZeitAlsEcht(final long zeit) {
		return (((zeit - ZeitSimulator.zeitpunktBeginnVirtuell) * 1000) / ZeitSimulator.multiplikator)
				+ ZeitSimulator.zeitpunktBeginnEcht;
	}

	/**
	 * konvertiert einen ZeitPunkt in reeller Zeit in virtuelle
	 *
	 * @param zeit
	 *            Reelle Zeit
	 * @return virtuelle Zeit
	 */
	static long getEchteZeitAlsVirtuell(final long zeit) {
		return (((zeit - ZeitSimulator.zeitpunktBeginnEcht) * ZeitSimulator.multiplikator) / 1000)
				+ ZeitSimulator.zeitpunktBeginnVirtuell;
	}

	/**
	 * konvertiert einen Zeitintervall in virtueller Zeit in reelle
	 *
	 * @param zeit
	 *            Virtueller Zeitintervall
	 * @return reeller Zeitintervall
	 */
	static long getVirtuelleZeitIntervallAlsEcht(final long zeit) {
		return (zeit * 1000) / ZeitSimulator.multiplikator;
	}

	/**
	 * Konvertiert einen Zeitintervall in reellen Zeit in virtuelle
	 *
	 * @param zeit
	 *            Reeller Zeitintervall
	 * @return Virtueller Zeitintervall
	 */
	static long getEchterZeitIntervallAlsVirtuell(final long zeit) {
		return (zeit * ZeitSimulator.multiplikator) / 1000;
	}

	/**
	 * Plant ein Ereignis ein Simmulation muss laufen, erst dann kann man
	 * Ereignisse planen
	 *
	 * @param erg
	 *            Ereignis
	 * @param zeitpunktVirtuell
	 *            Zeit
	 */
	static synchronized void schedule(final Ereignis erg, final long zeitpunktVirtuell) {
		final long weckZeitpunkt = ZeitSimulator.getVirtuelleZeitIntervallAlsEcht(
				zeitpunktVirtuell - ZeitSimulator.zeitpunktBeginnVirtuell) + ZeitSimulator.zeitpunktBeginnEcht;
		if (ZeitSimulator.lauft) {
			ZeitSimulator.wecker.setWecker(erg, weckZeitpunkt);
		}
	}

	/**
	 * Plant ein Ereignis ein Simmulation muss laufen, erst dann kann man
	 * Ereignisse planen
	 *
	 * @param erg
	 *            Ereignis
	 * @param zeitpunktVirtuellstring
	 *            Zeit im Stringformat
	 */
	static synchronized void schedule(final Ereignis erg, final String zeitpunktVirtuellstring) {
		try {

			final SimpleDateFormat dateFormat = new SimpleDateFormat(DUAKonstanten.ZEIT_FORMAT_GENAU_STR);
			final long zeitpunktVirtuell = dateFormat.parse(zeitpunktVirtuellstring).getTime();
			final long weckZeitpunkt = ZeitSimulator.getVirtuelleZeitIntervallAlsEcht(
					zeitpunktVirtuell - ZeitSimulator.zeitpunktBeginnVirtuell) + ZeitSimulator.zeitpunktBeginnEcht;

			System.out.println("Ereignis eingeplant: " + dateFormat.format(new Date(weckZeitpunkt)) + ", Virtuell: "
					+ dateFormat.format(new Date(zeitpunktVirtuell)));
			System.out.println(erg);
			if (ZeitSimulator.lauft) {
				ZeitSimulator.wecker.setWecker(erg, weckZeitpunkt);
			}
		} catch (final Exception e) {
			e.printStackTrace();
		}
	}

	protected static long getSimTimeout() {
		return ZeitSimulator.simTimeout;
	}
}
